package com.cars.car4you.api;

import com.cars.car4you.dto.CarDto;
import com.cars.car4you.model.Car;
import com.cars.car4you.repository.CarRepository;
import com.cars.car4you.service.CarService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/car/api")
public class CarsController {

    @Autowired
    private CarRepository carRepository;

    @Autowired
    private CarService carService;

    @GetMapping("/{id}")
    public Car findOne(@PathVariable Long id) {
        return carRepository.findById(id)
                .orElseThrow(NullPointerException::new);
    }

    @GetMapping("/model/{model}")
    public List<Car> findByModel(@PathVariable String model) {
        return carRepository.findByModel(model);
    }

    @GetMapping("/search")
    public List<CarDto> findByCriterion(@RequestParam(value = "fuel_type", required = false) String fuelType,
                                        @RequestParam(value = "seats", required = false) Integer seats,
                                        @RequestParam(value = "doors", required = false) Integer doors,
                                        @RequestParam(value = "maximum_price", required = false) Integer maxPrice,
                                        @RequestParam(value = "max_years_old", required = false) Integer maxYearsOld,
                                        @RequestParam(value = "engine_power_from", required = false) Integer enginePowerFrom,
                                        @RequestParam(value = "engine_power_to", required = false) Integer enginePowerTo,
                                        @RequestParam(value = "body_type", required = false) String bodyType,
                                        @RequestParam(value = "gearbox", required = false) String gearbox,
                                        @RequestParam(value = "engine_power_to", required = false) String driveType) {

        return carService.searchForCars(fuelType, seats, doors, maxPrice, maxYearsOld, enginePowerFrom, enginePowerTo, bodyType, gearbox, driveType);
    }
}
