package com.cars.car4you.repository;

import com.cars.car4you.model.*;
import org.springframework.stereotype.Service;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

@Service
public class CarRepositoryImpl implements CustomCarRepository  {

    @PersistenceContext
    private EntityManager em;

    @Override
    public List<Car> findByCriterion(Integer maxPrice, Integer maxYearsOld,
                                     Integer seats, Integer doors, String bodyType,
                                     String fuelType, String transmission,
                                     Integer enginePowerFrom, Integer enginePowerTo,
                                     String driveType) {
        CriteriaBuilder criteriaBuilder = em.getCriteriaBuilder();
        CriteriaQuery<Car> cq = criteriaBuilder.createQuery(Car.class);


        Root<Car> car = cq.from(Car.class);
        List<Predicate> predicates = new ArrayList<>();

        LocalDateTime localDateTime = LocalDateTime.now();

        if(maxPrice != null) predicates.add(criteriaBuilder.lessThanOrEqualTo(car.get("price_to"), maxPrice));
        if(maxYearsOld != null) predicates.add(criteriaBuilder.greaterThanOrEqualTo(car.get("year"),localDateTime.getYear() - maxYearsOld - 1));
        if(enginePowerFrom != null) predicates.add(criteriaBuilder.greaterThanOrEqualTo(car.get("engine_power"), enginePowerFrom));
        if(enginePowerTo != null) predicates.add(criteriaBuilder.lessThanOrEqualTo(car.get("engine_power"), enginePowerTo));
        if(transmission != null && !transmission.isEmpty()) predicates.add(criteriaBuilder.like(car.get("transmission"), "%" + transmission + "%"));
        if(fuelType != null && !fuelType.isEmpty()) predicates.add(criteriaBuilder.like(car.get("fuel"), "%" + fuelType + "%"));
        if(driveType != null && !driveType.isEmpty()) predicates.add(criteriaBuilder.like(car.get("drive"), "%" + driveType + "%"));
        if(seats != null) predicates.add(criteriaBuilder.equal(car.get("seats"), seats));
        if(doors != null) predicates.add(criteriaBuilder.equal(car.get("doors"), doors));

        cq.where(predicates.toArray(new Predicate[0]));
        return em.createQuery(cq).getResultList();
    }
}
