package com.cars.car4you.service;

import com.cars.car4you.dto.CarDto;
import com.cars.car4you.model.Car;
import com.cars.car4you.repository.CarRepository;
import org.apache.commons.math3.util.Precision;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class CarService {

    @Autowired
    CarRepository carRepository;

    public CarDto mapCarModelToCarDto(Car car){
        CarDto carDto = new CarDto();

        // Parametry konieczne do przeniesienia z modelu Car (tabela car)
        carDto.setYear(car.getYear());
        carDto.setEngine_power(car.getEngine_power());
        carDto.setEngine_capacity(car.getEngine_capacity());
        carDto.setFuel(car.getFuel());
        carDto.setTransmission(car.getTransmission());
        carDto.setDrive(car.getDrive());
        carDto.setBody_type(car.getBody_type());
        carDto.setDoors(car.getDoors());
        carDto.setSeats(car.getSeats());

        // ceny
        carDto.setPrice_from(car.getPrice_from());
        carDto.setPrice_to(car.getPrice_to());
        carDto.setAverage(car.getAvarage().longValue());

        // Parametry konieczne do przepisania z CarDetails (cars_details)
        carDto.setBrand(car.getCarDetails().getBrand());
        carDto.setModel(car.getCarDetails().getModel());
        carDto.setVersion(car.getCarDetails().getVersion());

        // Jeśli w (cars_details) kolumny dodatkowe są wypełnione to aktualizujemy model o nowe dane
        if(car.getCarDetails().getPrice_from() != null ) {
            carDto.setPrice_from(car.getCarDetails().getPrice_from());
            carDto.setPrice_to(car.getCarDetails().getPrice_to());
            carDto.setAverage(car.getCarDetails().getAverage());
            carDto.setCar_cons(car.getCarDetails().getCar_cons());
            carDto.setCar_pros(car.getCarDetails().getCar_pros());
        }
        if(carDto.getRating() != null) carDto.setRating(car.getCarDetails().getRating());


        return carDto;
    }

    private List<CarDto> convertListToDtoList(List<Car> cars){
        return cars.stream()
                .map(this::mapCarModelToCarDto)
                .collect(Collectors.toList());
    }

    private List<CarDto> processList(List<CarDto> dtoList){

        for(CarDto c : dtoList){
            Double fuelEconomy;
            if(c.getFuel() == "Diesel"){
                fuelEconomy = c.getEngine_power()*0.052174; //typowe spalanie L/KM dla diesla
                if(fuelEconomy < 5) fuelEconomy = 5.1;
                if(fuelEconomy > 15 ) fuelEconomy = 14.9;
            }else{
                fuelEconomy = c.getEngine_power()*0.1; //typowe spalanie L/KM dla benzyny
                if(fuelEconomy < 5) fuelEconomy = 5.1;
                if(fuelEconomy > 15) fuelEconomy = 14.9;
            }
            Precision.round(fuelEconomy, 1);
            c.setFuelEconomy(fuelEconomy);
        }


        //uzupełniam brakujące oceny losowo
        Random generator = new Random();
        for(CarDto c : dtoList){
            if(c.getRating() == null){
                Double rating = generator.nextDouble()*2+3; //ocena nie mniejsza niz 3
                c.setRating(Precision.round(rating, 1));
            }
        }

        //szukam ogłoszeń z pros and cons
        List<CarDto> prosAndCons = new ArrayList<>();
        for(CarDto c : dtoList){
            if (c.getCar_pros() != null && c.getCar_cons() != null) {
                prosAndCons.add(c);
            }
        }

        //ustawiam cons and pros dla aut oktore ich nie mają według losowych isniejących opisów
        for(CarDto c : dtoList){
            if (c.getCar_pros() == null && c.getCar_cons() == null) {
                c.setCar_cons(prosAndCons.get(generator.nextInt(prosAndCons.size())).getCar_cons());
                c.setCar_pros(prosAndCons.get(generator.nextInt(prosAndCons.size())). getCar_pros());
            }
        }

        Collections.sort(dtoList, (c2, c1) -> Double.compare(c1.getRating(), c2.getRating())); //sortowanie malejąco

        return dtoList;
    }

    public List<CarDto> searchForCars(String fuelType, Integer seats, Integer doors, Integer maxPrice,
                                      Integer maxYearsOld, Integer enginePowerFrom, Integer enginePowerTo,
                                      String bodyType, String gearbox, String driveType) {

        List<Car> rawCars = carRepository.findByCriterion(maxPrice, maxYearsOld, seats, doors, bodyType, fuelType, gearbox, enginePowerFrom, enginePowerTo, driveType);

        //jeżeli wyników jest mniej niż 3
        if(rawCars.size() <= 2){
            rawCars = carRepository.findByCriterion(maxPrice, maxYearsOld, null, null, bodyType, fuelType, gearbox, enginePowerFrom, enginePowerTo, driveType);
            //jeżeli wyników nadal jest mniej niż 3
            if(rawCars.size() <= 2){
                rawCars = carRepository.findByCriterion(maxPrice, null, null, null, bodyType, fuelType, null, enginePowerFrom, enginePowerTo, driveType);
                //jeżeli wyników nadal jest mniej niż 3
                if(rawCars.size() <= 2){
                    rawCars = carRepository.findByCriterion(maxPrice, null, null, null, bodyType, fuelType, null, null, null, null);
                }
            }
        }

        //jeżeli wyników jest wiecej niz 50 (to i tak jest już dużo)
        if(rawCars.size() > 10){
            List<Car> tmp = new ArrayList<>();
            for(int i = 0; i < 15; i++){
                tmp.add(rawCars.get(i));
            }
            rawCars = tmp;
        }

        List<CarDto> carDtos = convertListToDtoList(rawCars);
        return processList(carDtos);
    }

}
